package com.starbuds.server.service.statistics.dao;

import com.starbuds.server.common.pojo.db.RiskManageDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @Author lujiao
 * @DateTime 2021/7/5 18:52
 * @Description
 */
@Mapper
@Repository
public interface StatRiskManageDAO {

    /**
     * 插入数据
     *
     * @param po
     * @return
     */
    int insert(RiskManageDO po);

    /**
     * 获取数据的最大值
     *
     * @param fieldName 字段名称
     * @param dateType  日期类型
     * @return
     */
    RiskManageDO getGuinnessData(@Param("fieldName") String fieldName, @Param("dateType") Integer dateType);

    /**
     * 获取平均值
     *
     * @param startDate 起始日期
     * @param endDate   结束日期
     * @return
     */
    RiskManageDO getRiskAvgData(@Param("startDate") Integer startDate, @Param("endDate") Integer endDate);

    /**
     * 获取累计数据
     *
     * @param startDate
     * @param endDate
     * @return
     */
    RiskManageDO getRiskSummaryData(@Param("startDate") Integer startDate, @Param("endDate") Integer endDate);

    /**
     * 检查数据是否存在
     *
     * @param startDate
     * @param dateType
     * @return
     */
    Integer checkDataExist(@Param("startDate") Integer startDate, @Param("dateType") Integer dateType);

    /**
     * 更新
     *
     * @param po
     * @param startDate
     * @param dateType
     * @return
     */
    Integer update(@Param("po") RiskManageDO po, @Param("startDate") Integer startDate,
                   @Param("dateType") Integer dateType);

    /**
     * 更新状态
     *
     * @param riskId
     * @param status
     * @return
     */
    Integer updateStatus(@Param("riskId") Long riskId, @Param("status") Integer status);

    /**
     * 获取风控异常数据
     *
     * @return
     */
    List<RiskManageDO> getRiskData(@Param("pageCursor") Integer pageCursor, @Param("pageSize") Integer pageSize);

    /**
     * 获取风控异常数据总数
     *
     * @return
     */
    int getRiskDataCount();

    /**
     * 根据ID获取数据
     *
     * @param riskId
     * @return
     */
    RiskManageDO getRiskById(@Param("riskId") Long riskId);

    /**
     * 通过日期类型获取
     *
     * @param startDate
     * @param dateType
     * @return
     */
    RiskManageDO getRiskByDate(@Param("startDate") Integer startDate,
                               @Param("dateType") Integer dateType);
}
