package com.starbuds.server.service.statistics.impl;

import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.service.statistics.StatChannelService;
import com.starbuds.server.service.statistics.provider.StatChannelProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigDecimal;

/**
 * 平台数据统计服务实现
 *
 * @author Lay
 * @date 2020/2/5
 */
@Service(version = "1.0.0")
@Slf4j
public class StatChannelServiceImpl implements StatChannelService {

    @Autowired
    StatChannelProvider statChannelProvider;

    /**
     * 增加统计数
     *
     * @param channelId
     * @param statPlatformType
     * @param statValue
     */
    @Override
    public void addChannelStat(Long channelId, StatPlatformTypeEnum statPlatformType, BigDecimal statValue) {
        if (statPlatformType == null) {
            log.warn("未知的统计类型");
            return;
        }

        try {
            statChannelProvider.addStatAsync(channelId, statPlatformType.getValue(), statValue, System.currentTimeMillis());
        } catch (Exception e) {
            log.error("统计失败", e);
        }

    }
}
