package com.starbuds.server.service.statistics.impl;

import com.starbuds.server.common.constant.statistics.StatCommonBizSourceEnum;
import com.starbuds.server.common.pojo.db.StatCommonBizHisDO;
import com.starbuds.server.common.pojo.mq.statcommonbiz.QueueDivideCakeCostForkMsg;
import com.starbuds.server.common.pojo.mq.statcommonbiz.QueueDivideCakeGetForkMsg;
import com.starbuds.server.common.pojo.mq.statcommonbiz.QueueDivideCakeSendGiftTotalValueMsg;
import com.starbuds.server.common.pojo.mq.statcommonbiz.QueueDivideCakeSetupGiftTotalValueMsg;
import com.starbuds.server.service.statistics.provider.StatCommonBizHisProvider;
import com.starbuds.server.service.statistics.provider.StatCommonBizProvider;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

import static com.google.common.base.Preconditions.checkNotNull;
import static com.starbuds.server.common.constant.statistics.StatCommonBizTypeEnum.*;
import static com.starbuds.server.common.constant.statistics.StatTimeType.YEAR_MONTH_DAY_TYPES;

/**
 * 抽奖游戏餐叉统计
 *
 * @author aoe
 * @date 2021/12/20
 */
@Component
public class StatDivideCakeForkService {

    @Resource
    private StatCommonBizProvider statCommonBizProvider;

    @Resource
    private StatCommonBizHisProvider statCommonBizHisProvider;

    /**
     * 统计周年庆-瓜分蛋糕-用户获得餐叉
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void addGetForkStat(QueueDivideCakeGetForkMsg msg) {
        // 幂等 bizId + source
        createGetForkHis(msg);

        Long userId = msg.getUserId();
        Long count = NumberUtils.toLong("" + msg.getCount());
        Long timestamp = System.currentTimeMillis();
        statCommonBizProvider.addStat(ANNIVERSARY_DIVIDE_CAKE_ONE_USER_GET_FORK_COUNT, userId, count, timestamp, YEAR_MONTH_DAY_TYPES);
    }

    /**
     * 统计周年庆-瓜分蛋糕-用户消耗餐叉
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void addCostForkStat(QueueDivideCakeCostForkMsg msg) {
        // 幂等 bizId + source
        createCostForkHis(msg);

        Long userId = msg.getUserId();
        Long count = NumberUtils.toLong("" + msg.getCount());
        Long timestamp = System.currentTimeMillis();
        statCommonBizProvider.addStat(ANNIVERSARY_DIVIDE_CAKE_ONE_USER_COST_FORK_COUNT, userId, count, timestamp, YEAR_MONTH_DAY_TYPES);
    }

    /**
     * 统计周年庆-瓜分蛋糕-配置礼物总价值
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void addSetupGiftTotalValueStat(QueueDivideCakeSetupGiftTotalValueMsg msg) {
        // 幂等 bizId + source
        createSetupGiftTotalValueHis(msg);

        Long immutableId = 0L;
        Long giftTotalValue = msg.getGiftTotalValue();
        Long timestamp = System.currentTimeMillis();
        statCommonBizProvider.addStat(ANNIVERSARY_DIVIDE_CAKE_SETUP_GIFT_TOTAL_VALUE, immutableId, giftTotalValue, timestamp, YEAR_MONTH_DAY_TYPES);
    }

    /**
     * 统计周年庆-瓜分蛋糕-发放礼物总价值
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void addSendGiftTotalValueStat(QueueDivideCakeSendGiftTotalValueMsg msg) {
        // 幂等 bizId + source
        createSetupGiftTotalValueHis(msg);

        Long immutableId = 0L;
        Long totalValue = msg.getTotalValue();
        Long timestamp = System.currentTimeMillis();
        statCommonBizProvider.addStat(ANNIVERSARY_DIVIDE_CAKE_SEND_GIFT_TOTAL_VALUE, immutableId, totalValue, timestamp, YEAR_MONTH_DAY_TYPES);
    }

    @Transactional(rollbackFor = Exception.class)
    public void createGetForkHis(QueueDivideCakeGetForkMsg msg) {
        StatCommonBizHisDO bizHis = getBizHis(msg);
        bizHis = statCommonBizHisProvider.create(bizHis);
        checkNotNull(bizHis);
    }

    @Transactional(rollbackFor = Exception.class)
    public void createCostForkHis(QueueDivideCakeCostForkMsg msg) {
        StatCommonBizHisDO bizHis = getBizHis(msg);
        bizHis = statCommonBizHisProvider.create(bizHis);
        checkNotNull(bizHis);
    }

    @Transactional(rollbackFor = Exception.class)
    public void createSetupGiftTotalValueHis(QueueDivideCakeSetupGiftTotalValueMsg msg) {
        StatCommonBizHisDO bizHis = getBizHis(msg);
        bizHis = statCommonBizHisProvider.create(bizHis);
        checkNotNull(bizHis);
    }

    @Transactional(rollbackFor = Exception.class)
    public void createSetupGiftTotalValueHis(QueueDivideCakeSendGiftTotalValueMsg msg) {
        StatCommonBizHisDO bizHis = getBizHis(msg);
        bizHis = statCommonBizHisProvider.create(bizHis);
        checkNotNull(bizHis);
    }

    private StatCommonBizHisDO getBizHis(QueueDivideCakeGetForkMsg msg) {
        StatCommonBizHisDO po = new StatCommonBizHisDO();
        po.setBizId(msg.getOrderId());
        po.setSource(msg.getSource().value);
        return po;
    }

    private StatCommonBizHisDO getBizHis(QueueDivideCakeCostForkMsg msg) {
        StatCommonBizHisDO po = new StatCommonBizHisDO();
        po.setBizId(msg.getOrderId());
        po.setSource(msg.getSource().value);
        return po;
    }

    private StatCommonBizHisDO getBizHis(QueueDivideCakeSetupGiftTotalValueMsg msg) {
        StatCommonBizHisDO po = new StatCommonBizHisDO();
        po.setBizId(msg.getSnapId());
        po.setSource(StatCommonBizSourceEnum.ANNIVERSARY_DIVIDE_CAKE_SETUP_GIFT_TOTAL_VALUE.value);
        return po;
    }

    private StatCommonBizHisDO getBizHis(QueueDivideCakeSendGiftTotalValueMsg msg) {
        StatCommonBizHisDO po = new StatCommonBizHisDO();
        po.setBizId(msg.getOrderId());
        po.setSource(StatCommonBizSourceEnum.ANNIVERSARY_DIVIDE_CAKE_SEND_GIFT_TOTAL_VALUE.value);
        return po;
    }
}
