package com.starbuds.server.service.statistics.impl;

import com.starbuds.server.common.constant.statistics.StatRtcRoomTypeEnum;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.constant.statistics.StatUserTypeEnum;
import com.starbuds.server.common.service.statistics.StatRtcRoomService;
import com.starbuds.server.common.service.statistics.StatUserService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.statistics.provider.StatRtcRoomProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;

/**
 * 房间统计服务
 *
 * @author laoliu
 * @date 2021/09/02
 */
@Service(version = "1.0.0")
@Slf4j
public class StatRtcRoomServiceImpl implements StatRtcRoomService {

    @Resource
    StatRtcRoomProvider statRtcRoomProvider;

    @Reference(version = "1.0.0")
    private StatUserService statUserService;

    @Reference(version = "1.0.0")
    private StatRtcRoomService statRtcRoomService;

    /**
     * 增加统计数
     *
     * @param roomId
     * @param statRtcRoomType
     * @param statValue
     */
    @Override
    public void addRtcRoomStat(Long roomId, StatRtcRoomTypeEnum statRtcRoomType, BigDecimal statValue) {
        try {
            statRtcRoomProvider.addStatAsync(roomId, statRtcRoomType, statValue, System.currentTimeMillis());
        } catch (Exception e) {
            log.error("统计失败", e);
        }
    }

    /**
     * 获取房间统计数量
     *
     * @param roomId
     * @param statRtcRoomType
     * @param timeType
     * @param time
     * @return
     */
    @Override
    public Integer getRtcRoomStatCount(Long roomId, StatRtcRoomTypeEnum statRtcRoomType, Integer timeType, Long time) {
        Integer result = 0;
        try {
            result = statRtcRoomProvider.getRtcRoomStatCount(roomId, statRtcRoomType, timeType, time);
        } catch (Exception e) {
            log.error("统计失败", e);
        }
        return result;
    }

    /**
     * 更新房间统计数量
     *
     * @param roomId
     * @param statRtcRoomType
     * @param timeType
     * @param time
     * @return
     */
    @Override
    public void increaseRtcRoomStatCount(Long roomId, StatRtcRoomTypeEnum statRtcRoomType, Integer timeType, Long time) {
        try {
            statRtcRoomProvider.increaseRtcRoomStatCount(roomId, statRtcRoomType, timeType, time);
        } catch (Exception e) {
            log.error("统计失败", e);
        }
    }

    /**
     * 单独统计用户上麦
     * 2021-10-11从rtc移出
     *
     * @param roomId
     * @param userId
     */
    @Override
    public void statRtcRoomSitUpSeat(Long roomId, Long userId) {
        // 统计上麦用户数
        statUserService.addUserStat(userId, StatUserTypeEnum.SIT_SEAT_USERS.getValue(), BigDecimal.ONE);

        // 统计房间上麦用户
        if (statRtcRoomService.getRtcRoomStatCount(roomId, StatRtcRoomTypeEnum.RTC_ROOM_REQUEST_SEAT_USER, StatTimeType.YEAR_MONTH_DAY, DateUtil.getIntDate().longValue()) > 0) {

            statRtcRoomService.increaseRtcRoomStatCount(roomId, StatRtcRoomTypeEnum.RTC_ROOM_REQUEST_SEAT_USER, StatTimeType.YEAR_MONTH_DAY, DateUtil.getIntDate().longValue());
        } else {
            statRtcRoomService.addRtcRoomStat(roomId, StatRtcRoomTypeEnum.RTC_ROOM_REQUEST_SEAT_USER, BigDecimal.ONE);
        }

    }

}
