package com.starbuds.server.service.statistics.impl;

import com.starbuds.server.common.pojo.db.StatUserDO;
import com.starbuds.server.common.service.statistics.StatUserService;
import com.starbuds.server.service.statistics.provider.StatUserKeepProvider;
import com.starbuds.server.service.statistics.provider.StatUserProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.common.utils.CollectionUtils;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;

/**
 * 用户统计服务实现
 *
 * @author Lay
 * @date 2020/6/23
 */
@Service(version = "1.0.0")
@Slf4j
public class StatUserServiceImpl implements StatUserService {

    @Resource
    StatUserProvider statUserProvider;

    @Resource
    StatUserKeepProvider statUserKeepProvider;

    /**
     * 增加统计数
     *
     * @param userId
     * @param statUserType
     * @param statValue
     */
    @Override
    public void addUserStat(Long userId, int statUserType, BigDecimal statValue) {
        try {
            statUserProvider.addStatAsync(userId, statUserType, statValue, System.currentTimeMillis());
        } catch (Exception e) {
            log.error("统计失败", e);
        }
    }

    /**
     * 获取数据统计
     *
     * @param userId
     * @param statUserType
     * @param statTimeType
     * @param statTime
     * @param endTime
     * @return
     */
    @Override
    public BigDecimal sumStatByTimeRange(Long userId, int statUserType, int statTimeType, Long statTime, Long endTime) {
        List<StatUserDO> list = statUserProvider.getList(Collections.singletonList(statUserType), statTime, endTime, statTimeType);
        if (CollectionUtils.isNotEmpty(list)) {
            BigDecimal sumVal = BigDecimal.ZERO;
            for (StatUserDO statUser : list) {
                sumVal = sumVal.add(statUser.getStatValue());
            }
            return sumVal;
        }
        return BigDecimal.ZERO;
    }

    /**
     * 添加用户留存统计
     *
     * @param channelId
     * @param triggerDate
     * @param dateNum
     * @param dataType
     * @param dataValue
     */
    @Override
    public void addUserKeepStat(Long channelId, Long triggerDate, Long dateNum, Integer dataType, BigDecimal dataValue) {
        try {
            statUserKeepProvider.addStat(channelId, triggerDate, dateNum, dataType, dataValue);
        } catch (Exception e) {
            log.error("统计失败", e);
        }
    }
}
