package com.starbuds.server.service.statistics.provider;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.statistics.StatCommonBizTypeEnum;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.pojo.db.StatCommonBizDO;
import com.starbuds.server.service.statistics.dao.StatCommonBizDAO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 通用业务数据统计
 *
 * @author aoe
 * @date 2021/12/17
 */
@Component
@Slf4j
public class StatCommonBizProvider {

    @Resource
    private StatCommonBizDAO statCommonBizDAO;

    @Transactional(rollbackFor = Exception.class)
    public void addStat(StatCommonBizTypeEnum statType, Long bizId, Long statValue, Long timestamp, int[] statTimeTypes) {
        List<StatCommonBizDO> list = new ArrayList<>();
        for (int statTimeType : statTimeTypes) {
            StatCommonBizDO po = new StatCommonBizDO();
            po.setBizId(bizId);
            po.setStatValue(statValue);
            po.setStatType(statType.value);
            po.setStatTimeType(statTimeType);

            Date date = new Date(timestamp);
            SimpleDateFormat sdf = new SimpleDateFormat(StatTimeType.getTimeTypeFormat(statTimeType));
            po.setStatTime(Convert.toLong(sdf.format(date)));
            list.add(po);
        }
        statCommonBizDAO.batchInsertOrUpdate(list);
    }
}
