package com.starbuds.server.service.statistics.provider;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.statistics.StatRewardFeedGiftTypeEnum;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.pojo.db.StatRewardFeedGiftDO;
import com.starbuds.server.service.statistics.dao.StatRewardFeedGiftDAO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


/**
 * 送礼物-打赏动态数据统计
 *
 * @author aoe
 * @date 2021/11/17
 */
@Component
@Slf4j
public class StatRewardFeedGiftProvider {

    @Resource
    private StatRewardFeedGiftDAO statRewardFeedGiftDAO;

    /**
     * 添加礼物打赏统计
     *
     * @param statRewardFeedGiftType
     * @param feedId
     * @param statValue
     * @param timestamp
     */
    @Transactional(rollbackFor = Exception.class)
    public void addStat(StatRewardFeedGiftTypeEnum statRewardFeedGiftType, Long feedId, Long statValue, Long timestamp, int[] statTimeTypes) {
        if (statRewardFeedGiftType == null || feedId == null || statValue == null || timestamp == null) {
            log.error("送礼物-打赏动态数据统计收到数据异常 statRewardFeedGiftType={} feedId={} statValue={} timestamp={}",
                    statRewardFeedGiftType, feedId, statValue, timestamp);
            return;
        }

        List<StatRewardFeedGiftDO> list = new ArrayList<>();
        for (int statTimeType : statTimeTypes) {
            StatRewardFeedGiftDO po = new StatRewardFeedGiftDO();
            po.setStatValue(statValue);
            po.setStatType(statRewardFeedGiftType.getValue());
            po.setBizId(feedId);
            po.setStatTimeType(statTimeType);

            Date date = new Date(timestamp);
            SimpleDateFormat sdf = new SimpleDateFormat(StatTimeType.getTimeTypeFormat(statTimeType));
            po.setStatTime(Convert.toLong(sdf.format(date)));
            list.add(po);
        }
        statRewardFeedGiftDAO.batchInsertOrUpdate(list);
    }

}
