package com.starbuds.server.service.statistics.provider;

import com.starbuds.server.common.constant.statistics.RiskManageDateTypeEnum;
import com.starbuds.server.common.constant.statistics.RiskManageStatusEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.RiskManageDO;
import com.starbuds.server.service.statistics.dao.StatRiskManageDAO;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Author lujiao
 * @DateTime 2021/7/6 15:01
 * @Description
 */
@Component
public class StatRiskManageProvider {
    @Resource
    StatRiskManageDAO statRiskManageDAO;

    /**
     * 插入数据
     *
     * @param riskManageDO
     * @return
     */
    public void insert(RiskManageDO riskManageDO) {
        statRiskManageDAO.insert(riskManageDO);
    }

    /**
     * 获取区间时间范围内的平均数值
     *
     * @return
     */
    public RiskManageDO getRiskAvgData(Integer startDate, Integer endDate) {
        return statRiskManageDAO.getRiskAvgData(startDate, endDate);
    }

    /**
     * 检查数据是否已经存在
     *
     * @param dateInt
     * @param day
     * @return
     */
    public boolean checkExist(Integer dateInt, RiskManageDateTypeEnum day) {
        return statRiskManageDAO.checkDataExist(dateInt, day.getCode()) > 0;
    }

    /**
     * 获取区间时间范围内的总数值
     *
     * @param startDate
     * @param endDate
     * @return
     */
    public RiskManageDO getRiskSummary(Integer startDate, Integer endDate) {
        RiskManageDO res = statRiskManageDAO.getRiskSummaryData(startDate, endDate);
        if (res == null) {
            res = new RiskManageDO();
        }
        return res;
    }

    /**
     * 更新
     *
     * @param riskManageDO
     * @param startDate
     * @param dateType
     * @return
     */
    public Integer update(RiskManageDO riskManageDO, Integer startDate, RiskManageDateTypeEnum dateType) {
        return statRiskManageDAO.update(riskManageDO, startDate, dateType.getCode());
    }

    /**
     * 更新状态
     *
     * @param riskId
     * @param statusEnum
     * @return
     */
    public Integer updateStatus(Long riskId, RiskManageStatusEnum statusEnum) {
        return statRiskManageDAO.updateStatus(riskId, statusEnum.getCode());
    }

    /**
     * 获取该字段的吉尼斯数据
     *
     * @param fieldName
     * @param dateType  日期类型
     * @return
     */
    public RiskManageDO getGuinnessDate(String fieldName, Integer dateType) {
        return statRiskManageDAO.getGuinnessData(fieldName, dateType);
    }

    /**
     * 获取风控异常数据
     *
     * @return
     */
    public PageList<RiskManageDO> getRiskData(Integer pageIndex, Integer pageSize) {
        pageSize = pageSize / 2;
        PageList<RiskManageDO> page = new PageList<>(pageIndex, pageSize);
        page.setList(statRiskManageDAO.getRiskData(page.getPageCursor(), pageSize));
        page.setTotal(statRiskManageDAO.getRiskDataCount() * 2);
        return page;
    }

    public RiskManageDO getRiskById(Long riskId) {
        return statRiskManageDAO.getRiskById(riskId);
    }

    /**
     * 通过日期获取数据
     *
     * @param intDate
     * @param day
     * @return
     */
    public RiskManageDO getRiskByDate(Integer intDate, RiskManageDateTypeEnum day) {
        return statRiskManageDAO.getRiskByDate(intDate, day.getCode());
    }
}
