package com.starbuds.server.service.statistics.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.db.StatRoomFlowDO;
import com.starbuds.server.common.pojo.statistics.qo.StatRoomFlowPageListQO;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.statistics.dao.StatRoomFlowDAO;

import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 房间消费统计-房间里用户消费统计
 */
@Component
@Slf4j
public class StatRoomFlowProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private StatRoomFlowDAO statRoomFlowDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public StatRoomFlowDO create(StatRoomFlowDO po) {
        po.setHisId(identityService.getId());
        Long timestamp = System.currentTimeMillis();

        po.setCreateTime(timestamp);
        po.setCreateDate(DateUtil.getIntDate());
        boolean flag = statRoomFlowDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("房间消费统计-房间里用户消费统计创建成功 {}", json);
            return po;
        }
        log.warn("房间消费统计-房间里用户消费统计创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<StatRoomFlowDO> list) {
        boolean flag = statRoomFlowDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(StatRoomFlowDO po) {
        boolean flag = statRoomFlowDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(StatRoomFlowDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = statRoomFlowDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public StatRoomFlowDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return statRoomFlowDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return statRoomFlowDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<StatRoomFlowDO> getPageList(StatRoomFlowPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<StatRoomFlowDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(statRoomFlowDAO.selectPageList(qw));
        page.setTotal(statRoomFlowDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, StatRoomFlowDO> getMapByIds(Collection ids) {
        return statRoomFlowDAO.selectMapByIds(ids);
    }

    // 手动编码开始

    /**
     * 统计
     *
     * @param roomId
     * @param userId
     * @param amount
     */
    @Transactional(rollbackFor = Exception.class)
    public void addStat(Long roomId, Long userId, BigDecimal amount, Integer isBag, Integer isLuck) {
        StatRoomFlowDO platformFlowDO = new StatRoomFlowDO();
        platformFlowDO.setRoomId(roomId);
        platformFlowDO.setAmount(amount.longValue());
        platformFlowDO.setIsBag(isBag);
        platformFlowDO.setUserId(userId);
        platformFlowDO.setIsLuck(isLuck);
        create(platformFlowDO);
    }
}