package com.starbuds.server.service.statistics.provider;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.statistics.StatRtcRoomTypeEnum;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.pojo.db.StatRtcRoomDO;
import com.starbuds.server.service.statistics.dao.StatRtcRoomDAO;
import org.apache.logging.log4j.util.Strings;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/2/5
 */
@Component
public class StatRtcRoomProvider {

    @Resource
    StatRtcRoomDAO statRtcRoomDAO;

    @Async
    public void addStatAsync(Long channelId, StatRtcRoomTypeEnum statType, BigDecimal statValue, Long timestamp) {
        addStat(channelId, statType.getValue(), statValue, timestamp);
    }

    @Transactional(rollbackFor = Exception.class)
    public void addStat(Long roomId, Integer statType, BigDecimal statValue, Long timestamp) {
        List<StatRtcRoomDO> list = new ArrayList<>();
        // 分别构造年月日时分的数据

        // 年
        list.add(build(roomId, statType, statValue, StatTimeType.YEAR, timestamp));

        // 月
        list.add(build(roomId, statType, statValue, StatTimeType.YEAR_MONTH, timestamp));

        // 日
        list.add(build(roomId, statType, statValue, StatTimeType.YEAR_MONTH_DAY, timestamp));

        // 时
        list.add(build(roomId, statType, statValue, StatTimeType.YEAR_MONTH_DAY_HOUR, timestamp));

        // 分
        list.add(build(roomId, statType, statValue, StatTimeType.YEAR_MONTH_DAY_HOUR_MINUTE, timestamp));

        // 秒
        list.add(build(roomId, statType, statValue, StatTimeType.YEAR_MONTH_DAY_HOUR_MINUTE_SECOND, timestamp));

        statRtcRoomDAO.batchInsertOrUpdate(list);
    }

    /**
     * 获取房间统计数量
     *
     * @param roomId
     * @param statRtcRoomType
     * @param timeType
     * @param time
     * @return
     */
    public Integer getRtcRoomStatCount(Long roomId, StatRtcRoomTypeEnum statRtcRoomType, Integer timeType, Long time) {

        return Optional.ofNullable(statRtcRoomDAO.selectRtcRoomStatCount(roomId, statRtcRoomType.getValue(), timeType, time)).orElse(0);
    }

    /**
     * 更房间统计数量
     *
     * @param roomId
     * @param statRtcRoomType
     * @param timeType
     * @param time
     * @return
     */
    public void increaseRtcRoomStatCount(Long roomId, StatRtcRoomTypeEnum statRtcRoomType, Integer timeType, Long time) {

        statRtcRoomDAO.increaseRtcRoomStatCount(roomId, statRtcRoomType.getValue(), timeType, time);
    }

    private static StatRtcRoomDO build(Long roomId, Integer statType, BigDecimal statValue, Integer statTimeType, Long timestamp) {
        // 基础部分
        StatRtcRoomDO statRtcRoomDO = new StatRtcRoomDO();
        statRtcRoomDO.setRoomId(roomId);
        statRtcRoomDO.setStatType(statType);
        statRtcRoomDO.setStatValue(statValue);
        statRtcRoomDO.setStatTimeType(statTimeType);

        Date date = new Date(timestamp);
        SimpleDateFormat sdf = new SimpleDateFormat(getTimeTypeFormat(statTimeType));
        statRtcRoomDO.setStatTime(Convert.toLong(sdf.format(date)));

        return statRtcRoomDO;
    }

    private static String getTimeTypeFormat(Integer statTimeType) {
        switch (statTimeType) {
            case StatTimeType.YEAR:
                return "yyyy";
            case StatTimeType.YEAR_MONTH:
                return "yyyyMM";
            case StatTimeType.YEAR_MONTH_DAY:
                return "yyyyMMdd";
            case StatTimeType.YEAR_MONTH_DAY_HOUR:
                return "yyyyMMddHH";
            case StatTimeType.YEAR_MONTH_DAY_HOUR_MINUTE:
                return "yyyyMMddHHmm";
            case StatTimeType.YEAR_MONTH_DAY_HOUR_MINUTE_SECOND:
                return "yyyyMMddHHmmss";
            default:
                return Strings.EMPTY;
        }
    }

    public void saveAll(List<StatRtcRoomDO> statList) {
        statRtcRoomDAO.batchInsertOrUpdate(statList);
    }
}
