package com.starbuds.server.service.statistics.provider;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.pojo.db.StatSmsDeviceDO;
import com.starbuds.server.common.pojo.db.StatSmsIpDO;
import com.starbuds.server.service.statistics.dao.StatSmsDeviceDAO;
import com.starbuds.server.service.statistics.dao.StatSmsIpDAO;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Optional;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/6/14
 */
@Component
public class StatSmsProvider {

    @Resource
    StatSmsDeviceDAO statSmsDeviceDAO;

    @Resource
    StatSmsIpDAO statSmsIpDAO;

    @Async
    @Transactional(rollbackFor = Exception.class)
    public void addStatAsync(String ip, String deviceId) {
        Long curTime = System.currentTimeMillis();

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        int today = Convert.toInt(sdf.format(new Date()));

        // 更新统计
        StatSmsIpDO statSmsIp = new StatSmsIpDO();
        statSmsIp.setSendIp(ip);
        statSmsIp.setSendDate(today);
        statSmsIp.setSendCount(1);
        statSmsIp.setUpdateTime(curTime);
        statSmsIpDAO.insertOrUpdate(statSmsIp);

        StatSmsDeviceDO statSmsDevice = new StatSmsDeviceDO();
        statSmsDevice.setDeviceId(deviceId);
        statSmsDevice.setSendCount(1);
        statSmsDevice.setSendDate(today);
        statSmsDevice.setUpdateTime(curTime);
        statSmsDeviceDAO.insertOrUpdate(statSmsDevice);
    }

    public int getSmsIpCountDate(String ip, int sendDate) {
        return Optional.ofNullable(statSmsIpDAO.selectCountById(ip, sendDate)).orElse(0);
    }

    public int getSmsDeviceCountDate(String deviceId, int sendDate) {
        return Optional.ofNullable(statSmsDeviceDAO.selectCountById(deviceId, sendDate)).orElse(0);
    }
}
