package com.starbuds.server.service.statistics.provider;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.pojo.db.StatUserDO;
import com.starbuds.server.service.statistics.dao.StatUserDAO;
import org.apache.logging.log4j.util.Strings;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/6/23
 */
@Component
public class StatUserProvider {
    @Resource
    StatUserDAO statUserDAO;

    @Async
    @Transactional(rollbackFor = Exception.class)
    public void addStatAsync(Long userId, Integer statType, BigDecimal statValue, Long timestamp) {
        addStat(userId, statType, statValue, timestamp);
    }

    @Transactional(rollbackFor = Exception.class)
    public void addStat(Long userId, Integer statType, BigDecimal statValue, Long timestamp) {
        List<StatUserDO> list = new ArrayList<>();
        // 分别构造年月日时分的数据

        // 年
        list.add(build(userId, statType, statValue, StatTimeType.YEAR, timestamp));

        // 月
        list.add(build(userId, statType, statValue, StatTimeType.YEAR_MONTH, timestamp));

        // 日
        list.add(build(userId, statType, statValue, StatTimeType.YEAR_MONTH_DAY, timestamp));

        // 时
        list.add(build(userId, statType, statValue, StatTimeType.YEAR_MONTH_DAY_HOUR, timestamp));

        // 分
        list.add(build(userId, statType, statValue, StatTimeType.YEAR_MONTH_DAY_HOUR_MINUTE, timestamp));

        statUserDAO.batchInsertOrUpdate(list);
    }

    private static StatUserDO build(Long userId, Integer statType, BigDecimal statValue, Integer statTimeType, Long timestamp) {
        // 基础部分
        StatUserDO statUserDO = new StatUserDO();
        statUserDO.setUserId(userId);
        statUserDO.setStatType(statType);
        statUserDO.setStatValue(statValue);
        statUserDO.setStatTimeType(statTimeType);

        Date date = new Date(timestamp);
        SimpleDateFormat sdf = new SimpleDateFormat(getTimeTypeFormat(statTimeType));
        statUserDO.setStatTime(Convert.toLong(sdf.format(date)));

        return statUserDO;
    }

    public List<StatUserDO> getList(List<Integer> statTypes, Long startTime, Long endTime, Integer statTimeType) {
        return statUserDAO.selectByTypeAndTime(statTypes, startTime, endTime, statTimeType);
    }

    private static String getTimeTypeFormat(Integer statTimeType) {
        switch (statTimeType) {
            case StatTimeType.YEAR:
                return "yyyy";
            case StatTimeType.YEAR_MONTH:
                return "yyyyMM";
            case StatTimeType.YEAR_MONTH_DAY:
                return "yyyyMMdd";
            case StatTimeType.YEAR_MONTH_DAY_HOUR:
                return "yyyyMMddHH";
            case StatTimeType.YEAR_MONTH_DAY_HOUR_MINUTE:
                return "yyyyMMddHHmm";
            case StatTimeType.YEAR_MONTH_DAY_HOUR_MINUTE_SECOND:
                return "yyyyMMddHHmmss";
            default:
                return Strings.EMPTY;
        }
    }
}
