package com.starbuds.server.service.statistics.rocket;

import com.aliyun.openservices.ons.api.Consumer;
import com.aliyun.openservices.ons.api.ONSFactory;
import com.aliyun.openservices.ons.api.PropertyKeyConst;
import com.aliyun.openservices.ons.api.PropertyValueConst;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.service.statistics.rocket.listener.NormalMsgListener;
import com.starbuds.server.service.statistics.rocket.listener.StatisticsMsgListener;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.Properties;

/**
 * Class
 *
 * @author Lay
 * @date 2019/05/30
 */
@Component
@Slf4j
public class RocketConsumer implements DisposableBean {
    @Autowired
    RocketConfig rocketConfig;

    @Autowired
    StatisticsMsgListener statisticsMsgListener;

    @Autowired
    NormalMsgListener normalMsgListener;

    private Consumer consumer;

    @Override
    public void destroy() throws Exception {
        if (consumer != null) {
            consumer.shutdown();
        }
    }

    public void start() {
        log.info("RocketMQ 消费者初始化");
        Properties properties = rocketConfig.getProps();

        // 集群模式订阅，共同分担消息处理
        properties.put(PropertyKeyConst.MessageModel, PropertyValueConst.CLUSTERING);
        consumer = ONSFactory.createConsumer(properties);

        log.info("RocketMQ 消费者启动");
        consumer.subscribe(RocketTopic.TPC_STATISTICS, getSubscribeTags(Arrays.asList(
                RocketTag.STAT_PLATFORM
        )), statisticsMsgListener);
        consumer.subscribe(RocketTopic.TPC_NORMAL, getSubscribeTags(Arrays.asList(
                RocketTag.GIFT_PAID,
                RocketTag.RTC_GIFT_PAID,
                RocketTag.RTC_UNDER_MAI_GIFT_PAID,
                RocketTag.VOICE_GIFT_PAID,
                RocketTag.PRIVATE_CHAT_GIFT_PAID,
                RocketTag.RTC_SIT_UP_SEAT,
                RocketTag.USER_ONLINE_CHANGE,
                RocketTag.LIVE_USER_JOIN,
                RocketTag.LIVE_START,
                RocketTag.NOBLE_NEW,
                RocketTag.NOBLE_RENEW,
                RocketTag.NOBLE_UPGRADE,
                RocketTag.LOTTERY_WIN,
                RocketTag.LOTTERY_PAID,
                RocketTag.DIVIDE_CAKE_GET_FORK,
                RocketTag.DIVIDE_CAKE_COST_FORK,
                RocketTag.DIVIDE_CAKE_SETUP_GIFT_TOTAL_VALUE,
                RocketTag.DIVIDE_CAKE_SEND_GIFT_TOTAL_VALUE
        )), normalMsgListener);
        consumer.start();
    }

    private String getSubscribeTags(List<String> tags) {
        int size = tags.size();
        StringBuilder sbTags = new StringBuilder();
        for (int i = 0; i < size; i++) {
            sbTags.append(tags.get(i));
            if (i != size - 1) {
                sbTags.append("||");
            }
        }
        return sbTags.toString();
    }
}
