package com.starbuds.server.service.statistics.rocket.handler;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.PropertyKeyConst;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.data.DataMsgHitBussinessEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.media.GiftSceneTypeEnum;
import com.starbuds.server.common.constant.media.GiftTypeEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rtc.RtcBusinessTypeEnum;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.constant.statistics.StatRewardFeedGiftTypeEnum;
import com.starbuds.server.common.constant.statistics.StatRtcRoomTypeEnum;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.pojo.db.StatInviteDO;
import com.starbuds.server.common.pojo.finance.gift.GiftReceiver;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.common.service.data.DataMsgHisService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.media.LiveService;
import com.starbuds.server.common.service.statistics.StatRtcRoomService;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.statistics.dao.StatInviteDAO;
import com.starbuds.server.service.statistics.provider.*;
import com.starbuds.server.service.statistics.rocket.RocketConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import static com.starbuds.server.service.statistics.util.SpringContextUtil.isDev;

/**
 * 普通消息处理
 *
 * @author Lay
 * @date 2020/2/7
 */
@Component
@Slf4j
public class NormalMsgHandler {

    @Value("${spring.application.name}")
    String applicationName;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    LiveService liveService;

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Resource
    StatPlatformProvider statPlatformProvider;

    @Resource
    StatRtcRoomProvider statRtcRoomProvider;

    @Resource
    StatRoomFlowProvider statRoomFlowProvider;

    @Resource
    StatActiveUserProvider statActiveUserProvider;

    @Resource
    StatInviteDAO statInviteDAO;

    @Resource
    RocketConfig rocketConfig;

    @Reference(version = "1.0.0")
    DataMsgHisService dataMsgHisService;

    @Reference(version = "1.0.0")
    StatRtcRoomService statRtcRoomService;

    @Resource
    private StatRewardFeedGiftProvider statRewardFeedGiftProvider;

    /**
     * 礼物支付消息
     *
     * @param msg
     */
    public void handleGiftPaid(QueueGiftPaidMsg msg) {
        boolean isLotteryGift = GiftTypeEnum.LOTTERY.getValue() == msg.getGiftType();
        if (!msg.isFromBag()) {
            if (!isLotteryGift && WalletTypeEnum.STAR_DIAMOND == WalletTypeEnum.toEnum(msg.getGiftPriceType())) {
                BigDecimal amount = BigDecimal.valueOf(msg.getQuantity()).multiply(BigDecimal.valueOf(msg.getGiftPrice()));
                // 礼物消耗金币
                statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.GIFT_PAY_AMOUNT.getValue(), amount, msg.getTimestamp());
                // 场景
                GiftSceneTypeEnum sceneTypeEnum = GiftSceneTypeEnum.toEnum(msg.getGiftSceneType());
                switch (sceneTypeEnum) {
                    case Live:
                        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.GIFT_PAY_LIVE_AMOUNT.getValue(), amount, msg.getTimestamp());
                        break;
                    case SingleChat:
                        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.GIFT_PAY_SINGLE_CHAT_AMOUNT.getValue(), amount, msg.getTimestamp());
                        break;
                    default:
                        break;
                }

                // 幸运礼物消耗金币
                if (GiftTypeEnum.LUCKY == GiftTypeEnum.toEnum(msg.getGiftType())) {
                    statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.LUCKY_GIFT_PAY_AMOUNT.getValue(), amount, msg.getTimestamp());
                    if (msg.getWinAmount() != null && msg.getWinAmount() > 0) {
                        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.LUCKY_GIFT_WIN_AMOUNT.getValue(), BigDecimal.valueOf(msg.getWinAmount()), msg.getTimestamp());
                    }
                }

            }
        } else {
            if (!isLotteryGift && WalletTypeEnum.STAR_DIAMOND == WalletTypeEnum.toEnum(msg.getGiftPriceType())) {
                BigDecimal amount = BigDecimal.valueOf(msg.getQuantity()).multiply(BigDecimal.valueOf(msg.getGiftPrice()));
                // 礼物消耗金币
                statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.GIFT_PAY_AMOUNT_BAG.getValue(), amount, msg.getTimestamp());
                // 场景
                GiftSceneTypeEnum sceneTypeEnum = GiftSceneTypeEnum.toEnum(msg.getGiftSceneType());
                switch (sceneTypeEnum) {
                    case Live:
                        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.GIFT_PAY_LIVE_AMOUNT_BAG.getValue(), amount, msg.getTimestamp());
                        break;
                    case SingleChat:
                        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.GIFT_PAY_SINGLE_CHAT_AMOUNT_BAG.getValue(), amount, msg.getTimestamp());
                        break;
                    default:
                        break;
                }

                // 幸运礼物消耗金币
                if (GiftTypeEnum.LUCKY == GiftTypeEnum.toEnum(msg.getGiftType())) {
                    statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.LUCKY_GIFT_PAY_AMOUNT_BAG.getValue(), amount, msg.getTimestamp());
                    if (msg.getWinAmount() != null && msg.getWinAmount() > 0) {
                        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.LUCKY_GIFT_WIN_AMOUNT_BAG.getValue(), BigDecimal.valueOf(msg.getWinAmount()), msg.getTimestamp());
                    }
                }
            }
        }

        Long senderUserId = msg.getSenderUserId();
        Long receiverUserId = msg.getReceiverUserId();

        Long inviterUserId = userService.getInviterUserId(senderUserId);
        Long agentId = anchorService.getAgentId(receiverUserId);

        if (inviterUserId != null && agentId != null) {
            Integer quantity = msg.getQuantity();
            Long giftPrice = msg.getGiftPrice();

            StatInviteDO inviteDO = new StatInviteDO();
            inviteDO.setUserId(senderUserId);
            inviteDO.setInviterUserId(inviterUserId);
            inviteDO.setAgentId(agentId);
            inviteDO.setStatValue(giftPrice * quantity);
            statInviteDAO.insertOrUpdate(inviteDO);
        }
    }

    /**
     * 语聊礼物支付消息
     *
     * @param msg
     */
    public void handleRtcGiftPaid(QueueRtcGiftPaidMsg msg, String rocketTag) {
        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = rocketTag + "." + appName + "." + rockGropId + "." + orderId + "." + msg.getGiftId();
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }

        if (GiftTypeEnum.LOTTERY.getValue() != msg.getGiftType() && WalletTypeEnum.STAR_DIAMOND == WalletTypeEnum.toEnum(msg.getGiftPriceType())) {
            Map<Long, Integer> targetUserSeat = msg.getTargetUserSeat();
            if (CollectionUtils.isEmpty(targetUserSeat)) {
                return;
            }
            BigDecimal amount = BigDecimal.valueOf(msg.getQuantity()).multiply(BigDecimal.valueOf(msg.getGiftPrice()));
            int userNum = targetUserSeat.keySet().size();
            BigDecimal recordAmount = amount.multiply(BigDecimal.valueOf(userNum));

            // 语聊消耗金币
            statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.GIFT_PAY_RTC_ROOM_AMOUNT.getValue(), recordAmount, msg.getTimestamp());

            // 个播统计平台钻石流水
            if (RtcBusinessTypeEnum.RTC_SINGLE.equals(msg.getRoomBusinessType())) {
                statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.SINGLE_RTC_FLOW.getValue(), recordAmount, msg.getTimestamp());
            }

            // 语聊房统计
            statRtcRoomProvider.addStat(msg.getRoomId(), StatRtcRoomTypeEnum.SEND_STAR_DIAMOND.getValue(), recordAmount, msg.getTimestamp());

            // 房间用户消费统计
            statRoomFlowProvider.addStat(msg.getRoomId(), msg.getSenderUserId(), recordAmount, msg.isFromBag() ? BoolIntVal.True : BoolIntVal.False, msg.isFromLottery() ? BoolIntVal.True : BoolIntVal.False);

            // 平台用户消费统计
            statRoomFlowProvider.addStat(0L, msg.getSenderUserId(), recordAmount, msg.isFromBag() ? BoolIntVal.True : BoolIntVal.False, msg.isFromLottery() ? BoolIntVal.True : BoolIntVal.False);

            // 是否是背包礼物
            if (msg.isFromBag()) {
                statRtcRoomProvider.addStat(msg.getRoomId(), StatRtcRoomTypeEnum.BAG_GIFT_DIAMOND.getValue(), recordAmount, msg.getTimestamp());
            } else {
                // 这里是直刷流水统计
                if (msg.isFromLottery()) {
                    // 判断是否是幸运礼物
                    statRtcRoomProvider.addStat(msg.getRoomId(), StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND.getValue(), recordAmount, msg.getTimestamp());
                } else {
                    statRtcRoomProvider.addStat(msg.getRoomId(), StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND.getValue(), recordAmount, msg.getTimestamp());
                }
            }

            int roomFamilyUserCount = Optional.ofNullable(msg.getRoomFamilyUserCount()).orElse(0);
            if (userNum - roomFamilyUserCount < 0) {
                log.warn("收礼数量{}, 家庭用户数量{}, 出现负值，msg {}", userNum, roomFamilyUserCount, JSON.toJSONString(msg));
            }
            if (roomFamilyUserCount > 0) {
                BigDecimal familyRecordAmount = amount.multiply(BigDecimal.valueOf(roomFamilyUserCount));
                statRtcRoomProvider.addStat(msg.getRoomId(), StatRtcRoomTypeEnum.RECEIVE_STAR_DIAMOND_FAMILY_USER.getValue(), familyRecordAmount, msg.getTimestamp());
                if (msg.isFromBag()) {
                    statRtcRoomProvider.addStat(msg.getRoomId(), StatRtcRoomTypeEnum.BAG_GIFT_DIAMOND_FAMILY_USER.getValue(), familyRecordAmount, msg.getTimestamp());
                } else {
                    // 这里是直刷流水统计
                    if (msg.isFromLottery()) {
                        // 判断是否是幸运礼物
                        statRtcRoomProvider.addStat(msg.getRoomId(), StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND_FAMILY_USER.getValue(), familyRecordAmount, msg.getTimestamp());
                    } else {
                        statRtcRoomProvider.addStat(msg.getRoomId(), StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND_FAMILY_USER.getValue(), familyRecordAmount, msg.getTimestamp());
                    }
                }
            }
        }

        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);
    }

    /**
     * 处理语聊房麦下送礼物支付消息
     *
     * @param msg
     */
    public void handleRtcGiftUnderMaiPaid(QueueRtcGiftUnderMaiPaidMsg msg) {
        QueueRtcGiftPaidMsg rtcGiftPaidMsg = new QueueRtcGiftPaidMsg();
        BeanUtils.copyProperties(msg, rtcGiftPaidMsg);
        rtcGiftPaidMsg.setAllSeat(BoolIntVal.False);
        Map<Long, Integer> targetUserSeats = new HashMap<>();
        for (GiftReceiver receiver : msg.getReceiverUsers()) {
            targetUserSeats.put(receiver.getUserId(), -1);
        }
        rtcGiftPaidMsg.setTargetUserSeat(targetUserSeats);
        handleRtcGiftPaid(rtcGiftPaidMsg, RocketTag.RTC_UNDER_MAI_GIFT_PAID);
    }

    /**
     * 语音速配礼物支付消息
     *
     * @param msg
     */
    public void handleVoiceGiftPaid(QueueVoiceGiftPaidMsg msg) {
        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = RocketTag.VOICE_GIFT_PAID + "." + appName + "." + rockGropId + "." + orderId + "." + msg.getGiftId();
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }


        if (GiftTypeEnum.LOTTERY.getValue() != msg.getGiftType() && WalletTypeEnum.STAR_DIAMOND == WalletTypeEnum.toEnum(msg.getGiftPriceType())) {
            BigDecimal amount = BigDecimal.valueOf(msg.getQuantity()).multiply(BigDecimal.valueOf(msg.getGiftPrice()));
            if (msg.isFromBag()) {
                statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.GIFT_PAY_VOICE_CHAT_AMOUNT_BAG.getValue(), amount, msg.getTimestamp());
            } else {
                statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.GIFT_PAY_VOICE_CHAT_AMOUNT.getValue(), amount, msg.getTimestamp());
            }
        }


        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);

    }

    /**
     * 私聊礼物支付消息
     *
     * @param msg
     */
    public void handlePrivateChatGiftPaid(QueueBaseGiftPaidMsg msg) {
        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = RocketTag.PRIVATE_CHAT_GIFT_PAID + "." + appName + "." + rockGropId + "." + orderId + "." + msg.getGiftId();
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }

        if (GiftTypeEnum.LOTTERY.getValue() == msg.getGiftType() || WalletTypeEnum.STAR_DIAMOND != WalletTypeEnum.toEnum(msg.getGiftPriceType())) {
            return;
        }

        BigDecimal amount = BigDecimal.valueOf(msg.getQuantity()).multiply(BigDecimal.valueOf(msg.getGiftPrice()));
        long timestamp = msg.getTimestamp();
        if (msg.isFromBag()) {
            statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.GIFT_PAY_PRIVATE_CHAT_AMOUNT_BAG.getValue(), amount, timestamp);
        } else {
            statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.GIFT_PAY_PRIVATE_CHAT_AMOUNT.getValue(), amount, timestamp);
        }
        if (msg.isReward()) {
            // 打赏统计
            statRewardFeedGiftProvider.addStat(StatRewardFeedGiftTypeEnum.SEND_STAR_DIAMOND, msg.getFeedId(), amount.longValue(), timestamp, StatTimeType.ALL_TYPES);
            statRewardFeedGiftProvider.addStat(StatRewardFeedGiftTypeEnum.REWARD_COUNT, msg.getFeedId(), 1L, timestamp, StatTimeType.ALL_TYPES);
            // 打赏用户数统计（查询时根据bizId=用户的id去重）
            statRewardFeedGiftProvider.addStat(StatRewardFeedGiftTypeEnum.REWARD_USER_COUNT, msg.getSenderUserId(), 1L, timestamp, StatTimeType.YEAR_MONTH_DAY_TYPES);
        }

        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);
    }

    /**
     * 在线状态改变
     *
     * @param msg
     */
    public void handleUserOnlineChange(QueueUserOnlineChangeMsg msg) {
        Integer dateInt = com.starbuds.server.common.utils.DateUtil.getIntDate();
        // 排查Bug：用户在房间内，在线列表中看不到
        log.info("rocketmq-userStatusNotify-{}-{}-receive", msg.getUserId(), msg.getOnlineStatus());
        if (BoolIntVal.True == msg.getOnlineStatus()) {
            // 判断用户是否今日首次在线离线
            int todayTimes = userService.getOnlineTimesDayBefore(msg.getUserId(), null, msg.getTimestamp());
            log.info("用户上线 todayTimes: {} clientType: {}", todayTimes, msg.getClientType());
            if (todayTimes == 0) {
                // 增加日活
                statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.ACTIVE_USERS.getValue(), BigDecimal.ONE, msg.getTimestamp());

                ClientType clientType = ClientType.toEnum(msg.getClientType());
                switch (clientType) {
                    case iOS:
                        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.ACTIVE_USERS_IOS.getValue(), BigDecimal.ONE, msg.getTimestamp());
                        break;
                    case Android:
                        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.ACTIVE_USERS_ANDROID.getValue(), BigDecimal.ONE, msg.getTimestamp());
                        break;
                    default:
                        break;
                }

                // 增加活跃用户统计
                statActiveUserProvider.addStatActiveUser(msg.getChannelId(), Convert.toLong(DateUtil.format(new Date(userService.getCreateTime(msg.getUserId())), "yyyyMMdd")), Convert.toLong(DateUtil.format(new Date(msg.getTimestamp()), "yyyyMMdd")), 1);
            }

            if (isDev()) {
                log.info("[{}]用户上线了", msg.getUserId());
            }

            // 存放用户在线时间
            redisService.hashPut(RedisUserKeys.getPlatformUserLastOnline(dateInt), String.valueOf(msg.getUserId()), String.valueOf(System.currentTimeMillis()));

            // 判断最高在线用户
            Long onlineCount = redisService.hashSize(RedisUserKeys.getPlatformUserLastOnline(dateInt));
            if (onlineCount > Optional.ofNullable(redisService.get(RedisUserKeys.getPlatformOnlineUserMax(dateInt))).map(Convert::toLong).orElse(0L)) {
                redisService.set(RedisUserKeys.getPlatformOnlineUserMax(dateInt), String.valueOf(onlineCount));
            }
        } else {

            if (isDev()) {
                log.info("[{}]用户离线了", msg.getUserId());
            }

            if (redisService.hashExists(RedisUserKeys.getPlatformUserLastOnline(dateInt), String.valueOf(msg.getUserId()))) {
                // 该用户该次在线时长
                long duration = System.currentTimeMillis() - Long.parseLong(String.valueOf(redisService.hashGet(RedisUserKeys.getPlatformUserLastOnline(dateInt), String.valueOf(msg.getUserId()))));
                redisService.hashIncreaseLong(RedisUserKeys.getPlatformOnlineUserDuration(dateInt), String.valueOf(msg.getUserId()), duration);
                // 计算完删除该用户在线记录
                redisService.hashDel(RedisUserKeys.getPlatformUserLastOnline(dateInt), String.valueOf(msg.getUserId()));
            }
        }
    }

    /**
     * 用户进房
     *
     * @param msg
     */
    public void handleLiveUserJoin(QueueLiveUserJoinMsg msg) {
        // 判断是否今日首次进房
        int todayTimes = liveService.getLiveUserJoinDayBefore(msg.getJoinUserId(), null, BoolIntVal.True, msg.getTimestamp());
        if (todayTimes == 0) {
            // 增加进房用户数
            statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.JOIN_ROOM_USERS.getValue(), BigDecimal.ONE, msg.getTimestamp());
        }
    }

    /**
     * 主播开播
     *
     * @param msg
     */
    public void handleLiveStart(QueueLiveStartMsg msg) {
        // 判断是否今日首次开播
        int todayTimes = liveService.getLiveTimesDayBefore(msg.getAnchorUserId(), msg.getTimestamp());
        if (todayTimes == 0) {
            // 增加开播主播数
            statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.LIVE_ANCHORS.getValue(), BigDecimal.ONE, msg.getTimestamp());
            // 判断是否当日开播
            Long authTime = anchorService.getAnchorAuthTime(msg.getAnchorUserId());
            if (authTime != null) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
                if (sdf.format(new Date(authTime)).equals(sdf.format(new Date(msg.getTimestamp())))) {
                    statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.LIVE_ANCHORS_AUTH_DAILY.getValue(), BigDecimal.ONE, msg.getTimestamp());
                }

            }
        }
    }

    /**
     * 新开贵族
     *
     * @param msg
     */
    public void handleNobleNew(QueueNobleNewMsg msg) {
        log.info("新开贵族 {}", msg.getNobleLevel());
        // 加入统计
        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.STAT_NOBLE_NEW.getValue(), BigDecimal.ONE, msg.getTimestamp());
        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.STAT_NOBLE_NEW.getValue() + msg.getNobleLevel(), BigDecimal.ONE, msg.getTimestamp());
    }

    /**
     * 续费贵族
     *
     * @param msg
     */
    public void handleNobleRenew(QueueNobleRenewalMsg msg) {
        log.info("续费贵族 {}", msg.getNobleLevel());
        // 加入统计
        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.STAT_NOBLE_RENEW.getValue(), BigDecimal.ONE, msg.getTimestamp());
        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.STAT_NOBLE_RENEW.getValue() + msg.getNobleLevel(), BigDecimal.ONE, msg.getTimestamp());
    }

    /**
     * 升级贵族
     *
     * @param msg
     */
    public void handleNobleUpgrade(QueueNobleUpgradeMsg msg) {
        log.info("升级 {}", msg.getNewNobleLevel());
        // 加入统计
        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.STAT_NOBLE_UPGRADE.getValue(), BigDecimal.ONE, msg.getTimestamp());
        statPlatformProvider.addStat(msg.getChannelId(), StatPlatformTypeEnum.STAT_NOBLE_UPGRADE.getValue() + msg.getNewNobleLevel(), BigDecimal.ONE, msg.getTimestamp());
    }

    /**
     * 抽奖
     *
     * @param lotteryMsg
     */
    public void handleLotteryWin(QueueLotteryMsg lotteryMsg) {
    }

    /**
     * 单独统计用户上麦
     * 2021-10-11从rtc移出
     */
    public void handleRtcRoomSitUpSeat(QueueRtcSitUpSeatMsg msg) {
        statRtcRoomService.statRtcRoomSitUpSeat(msg.getRoomId(), msg.getUserId());
    }
}
