package com.starbuds.server.service.statistics.rocket.listener;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.common.pojo.mq.statcommonbiz.QueueDivideCakeCostForkMsg;
import com.starbuds.server.common.pojo.mq.statcommonbiz.QueueDivideCakeGetForkMsg;
import com.starbuds.server.common.pojo.mq.statcommonbiz.QueueDivideCakeSendGiftTotalValueMsg;
import com.starbuds.server.common.pojo.mq.statcommonbiz.QueueDivideCakeSetupGiftTotalValueMsg;
import com.starbuds.server.service.statistics.impl.StatDivideCakeForkService;
import com.starbuds.server.service.statistics.rocket.handler.NormalMsgHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/2/7
 */
@Component
@Slf4j
public class NormalMsgListener implements MessageListener {

    @Resource
    NormalMsgHandler normalMsgHandler;

    @Resource
    private StatDivideCakeForkService statDivideCakeForkService;

    @Override
    public Action consume(Message message, ConsumeContext consumeContext) {
        try {
            log.info("收到普通队列消息 {} {}", message.getTopic(), message.getTag());
            switch (message.getTag()) {
                case RocketTag.GIFT_PAID:
                    normalMsgHandler.handleGiftPaid(JSON.parseObject(message.getBody(), QueueGiftPaidMsg.class));
                    break;
                case RocketTag.RTC_GIFT_PAID:
                    normalMsgHandler.handleRtcGiftPaid(JSON.parseObject(message.getBody(), QueueRtcGiftPaidMsg.class), RocketTag.RTC_GIFT_PAID);
                    break;
                case RocketTag.RTC_UNDER_MAI_GIFT_PAID:
                    // 语聊房麦下送礼物支付消息
                    normalMsgHandler.handleRtcGiftUnderMaiPaid(JSON.parseObject(message.getBody(), QueueRtcGiftUnderMaiPaidMsg.class));
                    break;
                case RocketTag.VOICE_GIFT_PAID:
                    normalMsgHandler.handleVoiceGiftPaid(JSON.parseObject(message.getBody(), QueueVoiceGiftPaidMsg.class));
                    break;
                case RocketTag.PRIVATE_CHAT_GIFT_PAID:
                    normalMsgHandler.handlePrivateChatGiftPaid(JSON.parseObject(message.getBody(), QueueBaseGiftPaidMsg.class));
                    break;
                case RocketTag.RTC_SIT_UP_SEAT:
                    normalMsgHandler.handleRtcRoomSitUpSeat(JSON.parseObject(message.getBody(), QueueRtcSitUpSeatMsg.class));
                    break;
                case RocketTag.USER_ONLINE_CHANGE:
                    normalMsgHandler.handleUserOnlineChange(JSON.parseObject(message.getBody(), QueueUserOnlineChangeMsg.class));
                    break;
                case RocketTag.LIVE_USER_JOIN:
                    normalMsgHandler.handleLiveUserJoin(JSON.parseObject(message.getBody(), QueueLiveUserJoinMsg.class));
                    break;
                case RocketTag.LIVE_START:
                    normalMsgHandler.handleLiveStart(JSON.parseObject(message.getBody(), QueueLiveStartMsg.class));
                    break;
                case RocketTag.NOBLE_NEW:
                    normalMsgHandler.handleNobleNew(JSON.parseObject(message.getBody(), QueueNobleNewMsg.class));
                    break;
                case RocketTag.NOBLE_RENEW:
                    normalMsgHandler.handleNobleRenew(JSON.parseObject(message.getBody(), QueueNobleRenewalMsg.class));
                    break;
                case RocketTag.NOBLE_UPGRADE:
                    normalMsgHandler.handleNobleUpgrade(JSON.parseObject(message.getBody(), QueueNobleUpgradeMsg.class));
                    break;
                case RocketTag.LOTTERY_WIN:
                    normalMsgHandler.handleLotteryWin(JSON.parseObject(message.getBody(), QueueLotteryMsg.class));
                    break;
                case RocketTag.DIVIDE_CAKE_GET_FORK:
                    statDivideCakeForkService.addGetForkStat(JSON.parseObject(message.getBody(), QueueDivideCakeGetForkMsg.class));
                    break;
                case RocketTag.DIVIDE_CAKE_COST_FORK:
                    statDivideCakeForkService.addCostForkStat(JSON.parseObject(message.getBody(), QueueDivideCakeCostForkMsg.class));
                    break;
                case RocketTag.DIVIDE_CAKE_SETUP_GIFT_TOTAL_VALUE:
                    statDivideCakeForkService.addSetupGiftTotalValueStat(JSON.parseObject(message.getBody(), QueueDivideCakeSetupGiftTotalValueMsg.class));
                    break;
                case RocketTag.DIVIDE_CAKE_SEND_GIFT_TOTAL_VALUE:
                    statDivideCakeForkService.addSendGiftTotalValueStat(JSON.parseObject(message.getBody(), QueueDivideCakeSendGiftTotalValueMsg.class));
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            log.error(String.format("消息处理失败-%s-%s", message.getTag(), JSON.parseObject(message.getBody(), JSONObject.class)), e);
        }
        return Action.CommitMessage;
    }
}
